@@ -1,11 +1,9 @@
-use std::cell::{Cell, RefCell};
-
 use cairo::{self, ImageSurface};
 use cssparser::{CowRcStr, Parser, Token};
+use markup5ever::local_name;
 
-use crate::attributes::Attribute;
 use crate::drawing_ctx::DrawingCtx;
-use crate::error::{NodeError, ValueErrorKind};
+use crate::error::{AttributeResultExt, ValueErrorKind};
 use crate::node::{NodeResult, NodeTrait, RsvgNode};
 use crate::parsers::{self, Parse, ParseValue};
 use crate::property_bag::PropertyBag;
@@ -36,52 +34,44 @@ enum Operator {
 /// The `feComposite` filter primitive.
 pub struct Composite {
     base: PrimitiveWithInput,
-    in2: RefCell<Option<Input>>,
-    operator: Cell<Operator>,
-    k1: Cell<f64>,
-    k2: Cell<f64>,
-    k3: Cell<f64>,
-    k4: Cell<f64>,
+    in2: Option<Input>,
+    operator: Operator,
+    k1: f64,
+    k2: f64,
+    k3: f64,
+    k4: f64,
 }
 
-impl Composite {
+impl Default for Composite {
     /// Constructs a new `Composite` with empty properties.
     #[inline]
-    pub fn new() -> Composite {
+    fn default() -> Composite {
         Composite {
             base: PrimitiveWithInput::new::<Self>(),
-            in2: RefCell::new(None),
-            operator: Cell::new(Operator::Over),
-            k1: Cell::new(0f64),
-            k2: Cell::new(0f64),
-            k3: Cell::new(0f64),
-            k4: Cell::new(0f64),
+            in2: None,
+            operator: Operator::Over,
+            k1: 0.0,
+            k2: 0.0,
+            k3: 0.0,
+            k4: 0.0,
         }
     }
 }
 
 impl NodeTrait for Composite {
-    fn set_atts(&self, node: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
-        self.base.set_atts(node, pbag)?;
+    impl_node_as_filter!();
+
+    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+        self.base.set_atts(parent, pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::In2 => {
-                    self.in2.replace(Some(Input::parse(Attribute::In2, value)?));
-                }
-                Attribute::Operator => self.operator.set(attr.parse(value)?),
-                Attribute::K1 => self.k1.set(
-                    parsers::number(value).map_err(|err| NodeError::attribute_error(attr, err))?,
-                ),
-                Attribute::K2 => self.k2.set(
-                    parsers::number(value).map_err(|err| NodeError::attribute_error(attr, err))?,
-                ),
-                Attribute::K3 => self.k3.set(
-                    parsers::number(value).map_err(|err| NodeError::attribute_error(attr, err))?,
-                ),
-                Attribute::K4 => self.k4.set(
-                    parsers::number(value).map_err(|err| NodeError::attribute_error(attr, err))?,
-                ),
+                local_name!("in2") => self.in2 = Some(Input::parse(attr, value)?),
+                local_name!("operator") => self.operator = attr.parse(value)?,
+                local_name!("k1") => self.k1 = parsers::number(value).attribute(attr)?,
+                local_name!("k2") => self.k2 = parsers::number(value).attribute(attr)?,
+                local_name!("k3") => self.k3 = parsers::number(value).attribute(attr)?,
+                local_name!("k4") => self.k4 = parsers::number(value).attribute(attr)?,
                 _ => (),
             }
         }
@@ -147,7 +137,7 @@ impl Filter for Composite {
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
         let input = self.base.get_input(ctx, draw_ctx)?;
-        let input_2 = ctx.get_input(draw_ctx, self.in2.borrow().as_ref())?;
+        let input_2 = ctx.get_input(draw_ctx, self.in2.as_ref())?;
         let bounds = self
             .base
             .get_bounds(ctx)
@@ -172,27 +162,22 @@ impl Filter for Composite {
             input.surface().surface_type()
         };
 
-        let output_surface = if self.operator.get() == Operator::Arithmetic {
+        let output_surface = if self.operator == Operator::Arithmetic {
             let mut output_surface = ImageSurface::create(
                 cairo::Format::ARgb32,
                 input.surface().width(),
                 input.surface().height(),
             )?;
 
-            let k1 = self.k1.get();
-            let k2 = self.k2.get();
-            let k3 = self.k3.get();
-            let k4 = self.k4.get();
-
             composite_arithmetic(
                 input.surface(),
                 input_2.surface(),
                 &mut output_surface,
                 bounds,
-                k1,
-                k2,
-                k3,
-                k4,
+                self.k1,
+                self.k2,
+                self.k3,
+                self.k4,
             );
 
             output_surface
@@ -209,14 +194,14 @@ impl Filter for Composite {
             cr.clip();
 
             input.surface().set_as_source_surface(&cr, 0f64, 0f64);
-            cr.set_operator(self.operator.get().into());
+            cr.set_operator(self.operator.into());
             cr.paint();
 
             output_surface
         };
 
         Ok(FilterResult {
-            name: self.base.result.borrow().clone(),
+            name: self.base.result.clone(),
             output: FilterOutput {
                 surface: SharedImageSurface::new(output_surface, surface_type)?,
                 bounds,
