@@ -1,25 +1,19 @@
-use crate::attributes::Attribute;
+use markup5ever::local_name;
+
 use crate::node::{NodeResult, NodeTrait, NodeType, RsvgNode};
 use crate::property_bag::PropertyBag;
 use crate::text::NodeChars;
 
-use std::cell::RefCell;
-
 /// Represents a <style> node.
 ///
 /// It does not render itself, and just holds CSS stylesheet information for the rest of
 /// the code to use.
+#[derive(Default)]
 pub struct NodeStyle {
-    type_: RefCell<Option<String>>,
+    type_: Option<String>,
 }
 
 impl NodeStyle {
-    pub fn new() -> NodeStyle {
-        NodeStyle {
-            type_: RefCell::new(None),
-        }
-    }
-
     pub fn get_css(&self, node: &RsvgNode) -> String {
         // FIXME: See these:
         //
@@ -30,19 +24,14 @@ impl NodeStyle {
         // "contentStyleType" attribute of the svg element, which in turn
         // defaults to "text/css".
 
-        let have_css = self
-            .type_
-            .borrow()
-            .as_ref()
-            .map(|t| t == "text/css")
-            .unwrap_or(true);
+        let have_css = self.type_.as_ref().map(|t| t == "text/css").unwrap_or(true);
 
         if have_css {
             node.children()
                 .into_iter()
                 .filter_map(|child| {
-                    if child.get_type() == NodeType::Chars {
-                        Some(child.with_impl(|chars: &NodeChars| chars.get_string()))
+                    if child.borrow().get_type() == NodeType::Chars {
+                        Some(child.borrow().get_impl::<NodeChars>().get_string())
                     } else {
                         None
                     }
@@ -55,10 +44,10 @@ impl NodeStyle {
 }
 
 impl NodeTrait for NodeStyle {
-    fn set_atts(&self, _: &RsvgNode, pbag: &PropertyBag<'_>) -> NodeResult {
+    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
         for (attr, value) in pbag.iter() {
-            if attr == Attribute::Type {
-                *self.type_.borrow_mut() = Some(value.to_string());
+            if attr == local_name!("type") {
+                self.type_ = Some(value.to_string());
             }
         }
 
