@@ -10,7 +10,7 @@ use crate::allowed_url::{AllowedUrl, Fragment};
 use crate::error::LoadingError;
 use crate::handle::LoadOptions;
 use crate::io::{self, BinaryData};
-use crate::node::{NodeType, RsvgNode};
+use crate::node::{NodeCascade, NodeType, RsvgNode};
 use crate::properties::ComputedValues;
 use crate::structure::{IntrinsicDimensions, NodeSvg};
 use crate::surface_utils::shared_surface::SharedImageSurface;
@@ -36,7 +36,11 @@ pub struct Svg {
 }
 
 impl Svg {
-    pub fn new(tree: RsvgNode, ids: HashMap<String, RsvgNode>, load_options: LoadOptions) -> Svg {
+    pub fn new(
+        mut tree: RsvgNode,
+        ids: HashMap<String, RsvgNode>,
+        load_options: LoadOptions,
+    ) -> Svg {
         let values = ComputedValues::default();
         tree.cascade(&values);
 
@@ -81,16 +85,15 @@ impl Svg {
     }
 
     pub fn lookup_image(&self, href: &str) -> Result<SharedImageSurface, LoadingError> {
-        dbg!(&self.load_options.base_url);
         self.images.borrow_mut().lookup(&self.load_options, href)
     }
 
     pub fn get_intrinsic_dimensions(&self) -> IntrinsicDimensions {
         let root = self.root();
+        let node_data = root.borrow();
 
-        assert!(root.get_type() == NodeType::Svg);
-
-        root.with_impl(|svg: &NodeSvg| svg.get_intrinsic_dimensions())
+        assert!(node_data.get_type() == NodeType::Svg);
+        node_data.get_impl::<NodeSvg>().get_intrinsic_dimensions()
     }
 }
 
