@@ -3,17 +3,17 @@ use encoding::label::encoding_from_whatwg_label;
 use encoding::DecoderTrap;
 use glib::IsA;
 use libc;
+use markup5ever::{local_name, LocalName};
 use std::collections::HashMap;
 use std::str;
 
 use crate::allowed_url::AllowedUrl;
-use crate::attributes::Attribute;
 use crate::create_node::create_node_and_register_id;
 use crate::css::CssRules;
 use crate::error::LoadingError;
 use crate::handle::LoadOptions;
 use crate::io::{self, get_input_stream_for_loading};
-use crate::node::{NodeType, RsvgNode};
+use crate::node::{NodeData, NodeType, RsvgNode};
 use crate::property_bag::PropertyBag;
 use crate::style::NodeStyle;
 use crate::svg::Svg;
@@ -94,7 +94,7 @@ impl XmlState {
         XmlState {
             tree_root: None,
             ids: Some(HashMap::new()),
-            css_rules: Some(CssRules::new()),
+            css_rules: Some(CssRules::default()),
             context_stack: vec![Context::Start],
             current_node: None,
             entities: HashMap::new(),
@@ -113,22 +113,21 @@ impl XmlState {
     pub fn steal_result(&mut self) -> Result<Svg, LoadingError> {
         match self.tree_root {
             None => Err(LoadingError::SvgHasNoElements),
+            Some(ref root) if root.borrow().get_type() == NodeType::Svg => {
+                let root = self.tree_root.take().unwrap();
+                let css_rules = self.css_rules.as_ref().unwrap();
 
-            Some(ref root) => {
-                if root.get_type() == NodeType::Svg {
-                    let root = self.tree_root.take().unwrap();
-
-                    root.set_styles_recursively(&root, self.css_rules.as_ref().unwrap());
-
-                    Ok(Svg::new(
-                        root,
-                        self.ids.take().unwrap(),
-                        self.load_options.clone(),
-                    ))
-                } else {
-                    Err(LoadingError::RootElementIsNotSvg)
+                for mut node in root.descendants() {
+                    node.borrow_mut().set_style(css_rules);
                 }
+
+                Ok(Svg::new(
+                    root,
+                    self.ids.take().unwrap(),
+                    self.load_options.clone(),
+                ))
             }
+            _ => Err(LoadingError::RootElementIsNotSvg),
         }
     }
 
@@ -182,7 +181,13 @@ impl XmlState {
         let context = self.context();
 
         match context {
-            Context::Start => panic!("characters: XML handler stack is empty!?"),
+            // This is character data before the first element, i.e. something like
+            //  <?xml version="1.0" encoding="UTF-8"?><svg xmlns="http://www.w3.org/2000/svg"/>
+            // ^ note the space here
+            // libxml2 is not finished reading the file yet; it will emit an error
+            // on its own when it finishes.  So, ignore this condition.
+            Context::Start => return,
+
             Context::ElementCreation => self.element_creation_characters(text),
             Context::XInclude(_) => (),
             Context::UnsupportedXIncludeChild => (),
@@ -255,11 +260,19 @@ impl XmlState {
         match name {
             "include" => self.xinclude_start_element(name, pbag),
             _ => {
+                let ids = self.ids.as_mut().unwrap();
+                let mut node = create_node_and_register_id(name, pbag, ids);
+
                 let parent = self.current_node.clone();
-                let node = self.create_node(parent.as_ref(), name, pbag);
-                if self.current_node.is_none() {
+                node.borrow_mut()
+                    .set_atts(parent.as_ref(), pbag, self.load_options.locale());
+
+                if let Some(mut parent) = parent {
+                    parent.append(node.clone());
+                } else {
                     self.set_root(&node);
                 }
+
                 self.current_node = Some(node);
 
                 Context::ElementCreation
@@ -270,10 +283,9 @@ impl XmlState {
     fn element_creation_end_element(&mut self) {
         let node = self.current_node.take().unwrap();
 
-        if node.get_type() == NodeType::Style {
-            let css_data = node.with_impl(|style: &NodeStyle| style.get_css(&node));
-
+        if node.borrow().get_type() == NodeType::Style {
             let css_rules = self.css_rules.as_mut().unwrap();
+            let css_data = node.borrow().get_impl::<NodeStyle>().get_css(&node);
 
             css_rules.parse(self.load_options.base_url.as_ref(), &css_data);
         }
@@ -282,55 +294,36 @@ impl XmlState {
     }
 
     fn element_creation_characters(&self, text: &str) {
-        let node = self.current_node.as_ref().unwrap();
-
         if text.len() != 0 {
             // When the last child is a Chars node we can coalesce
             // the text and avoid screwing up the Pango layouts
-            let chars_node = if let Some(child) = node
+            let chars_node = if let Some(child) = self
+                .current_node
+                .as_ref()
+                .unwrap()
                 .last_child()
-                .filter(|c| c.get_type() == NodeType::Chars)
+                .filter(|c| c.borrow().get_type() == NodeType::Chars)
             {
                 child
             } else {
-                let child = RsvgNode::new(
+                let child = RsvgNode::new(NodeData::new(
                     NodeType::Chars,
-                    Some(node),
+                    LocalName::from("rsvg-chars"),
                     None,
                     None,
                     Box::new(NodeChars::new()),
-                );
-                node.append(&child);
+                ));
+
+                let mut node = self.current_node.as_ref().unwrap().clone();
+                node.append(child.clone());
+
                 child
             };
 
-            chars_node.with_impl(|chars: &NodeChars| {
-                chars.append(text);
-            });
+            chars_node.borrow().get_impl::<NodeChars>().append(text);
         }
     }
 
-    fn create_node(
-        &mut self,
-        parent: Option<&RsvgNode>,
-        name: &str,
-        pbag: &PropertyBag,
-    ) -> RsvgNode {
-        let ids = self.ids.as_mut().unwrap();
-
-        let new_node = create_node_and_register_id(name, parent, pbag, ids);
-
-        if let Some(parent) = parent {
-            parent.append(&new_node);
-        }
-
-        new_node.set_atts(&new_node, pbag, self.load_options.locale());
-
-        new_node.set_overridden_properties();
-
-        new_node
-    }
-
     fn xinclude_start_element(&mut self, _name: &str, pbag: &PropertyBag) -> Context {
         let mut href = None;
         let mut parse = None;
@@ -338,9 +331,9 @@ impl XmlState {
 
         for (attr, value) in pbag.iter() {
             match attr {
-                Attribute::Href => href = Some(value),
-                Attribute::Parse => parse = Some(value),
-                Attribute::Encoding => encoding = Some(value),
+                local_name!("href") => href = Some(value),
+                ref n if *n == LocalName::from("parse") => parse = Some(value),
+                local_name!("encoding") => encoding = Some(value),
                 _ => (),
             }
         }
@@ -390,7 +383,13 @@ impl XmlState {
     }
 
     fn xinclude_fallback_characters(&mut self, ctx: &XIncludeContext, text: &str) {
-        if ctx.need_fallback {
+        if ctx.need_fallback && self.current_node.is_some() {
+            // We test for is_some() because with a bad "SVG" file like this:
+            //
+            //    <xi:include href="blah"><xi:fallback>foo</xi:fallback></xi:include>
+            //
+            // at the point we get "foo" here, there is no current_node because
+            // no nodes have been created before the xi:include.
             self.element_creation_characters(text);
         }
     }
